var Backend, Sqlite3, SqliteBackend;
var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
  for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
  function ctor() { this.constructor = child; }
  ctor.prototype = parent.prototype;
  child.prototype = new ctor;
  child.__super__ = parent.prototype;
  return child;
}, __slice = Array.prototype.slice, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
Sqlite3 = require('sqlite3').verbose();
Backend = require('./core');
exports.create = function(options) {
  return new SqliteBackend(options || {});
};
SqliteBackend = (function() {
  __extends(SqliteBackend, Backend);
  function SqliteBackend(options) {
    this.db = new Sqlite3.Database(options.path || ':memory:');
    SqliteBackend.__super__.constructor.call(this);
  }
  SqliteBackend.prototype.push = function(msg, cb) {
    var obj;
    obj = msg.data != null ? msg : {
      data: msg.toString()
    };
    obj.run_at || (obj.run_at = new Date);
    obj.retries || (obj.retries = 0);
    return this.db.run("INSERT INTO messages (data, run_at, retries) VALUES (?, ?, ?)", obj.data, this.dateToSql(obj.run_at), obj.retries, __bind(function() {
      var args;
      args = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
      this.events.emit('incoming');
      return typeof cb === "function" ? cb.apply(null, args) : void 0;
    }, this));
  };
  SqliteBackend.prototype.peek = function(cb) {
    var sql;
    sql = "SELECT rowid AS id, data, retries, run_at FROM messages      WHERE run_at <= ?      ORDER BY run_at,rowid LIMIT 1";
    return this.db.get(sql, this.dateToSql(), cb);
  };
  SqliteBackend.prototype.count = function(cb) {
    return this.db.get("SELECT COUNT(rowid) as count FROM messages", function(err, row) {
      if (err) {
        return cb(err);
      } else {
        return cb(null, row.count);
      }
    });
  };
  SqliteBackend.prototype.remove = function(id, cb) {
    return this.db.run("DELETE FROM messages WHERE rowid = ?", id, cb);
  };
  SqliteBackend.prototype.transaction = function(cb) {
    return this.db.serialize(cb);
  };
  SqliteBackend.prototype.reschedule = function(obj, time, cb) {
    obj.retries || (obj.retries = 0);
    obj.retries += 1;
    obj.run_at = this.rescheduledTime(time, obj.retries);
    return this.transaction(__bind(function() {
      this.remove(obj.id);
      return this.push(obj, cb);
    }, this));
  };
  SqliteBackend.prototype.rescheduledTime = function(time, retries) {
    var duration, seconds;
    time || (time = new Date);
    duration = Math.pow(retries, 4) + 5;
    seconds = time - -(duration * 1000);
    return new Date(seconds);
  };
  SqliteBackend.prototype.dateToSql = function(date) {
    date || (date = new Date);
    return date.toISOString().replace('T', ' ').replace(/\..*/, '');
  };
  SqliteBackend.prototype.setup = function(cb) {
    return this.db.serialize(__bind(function() {
      this.db.run("CREATE TABLE IF NOT EXISTS messages (        data TEXT,        retries INTEGER,        run_at DATETIME DEFAULT CURRENT_TIMESTAMP)", __bind(function(err) {
        if (err) {
          if (typeof cb === "function") {
            cb(err);
          }
          return this.events.emit('error', err);
        }
      }, this));
      return this.db.run("CREATE INDEX IF NOT EXISTS messages_by_run_at ON messages (run_at)", __bind(function(err) {
        if (err) {
          if (typeof cb === "function") {
            cb(err);
          }
          return this.events.emit('error', err);
        }
      }, this));
    }, this));
  };
  return SqliteBackend;
})();